/***********************************************************************************/
/*                                                                                 */
/*                     Camera Entity Class Definition                              */
/*                                                                                 */
/***********************************************************************************/

/*
The camera entity class is a derivative of the entity class, used within
the scene to determine which part of the scene should be drawn. The camera 
is made aware of another entity which is deemed the target, and will update 
its position to track the target on each frame. The camera entity is also
able to interpolate smoothly between two positions.
*/


#include <iostream>
#include "cameraEntity.h"
#include "math.h"

using namespace std;

CameraEntity::CameraEntity(float x, float y) : DynamicEntity(x, y)
{
	startPos.x = x;
	startPos.y = y;
	status = 0.0f;
	targetEnt = NULL;

	offset.x=0.0f;
	offset.y=2.0f;

}

void CameraEntity::updateEntity(void)
{
	if(targetEnt == NULL)
		return;

	previousPos = getPosition();

	/*--------- Update position based on target entity-------*/

	if(status == 0.0f) //static camera
	{
		if( (targetEnt->getPosX() - startPos.x) > 9
		||  (targetEnt->getPosX() - startPos.x) < -9)
			status = 0.01f;

	}
	else if(status < 1.0f) //interpolating camera
	{
		float t = status;

		//cubic interpolation
		float v2 = -2*(t*t*t) + 3*(t*t); // -2t^3 + 3t^2
		float v1 = 2*(t*t*t) - 3*(t*t) + 1; // 2t^3 - 3t^2 + 1

		pos.x = ( (v1 * startPos.x) + (v2 * (targetEnt->getPosX()+offset.x) ) );
		pos.y = ( (v1 * startPos.y) + (v2 * (targetEnt->getPosY()+offset.y) ) );

		status+=0.01f;

	}
	else //following camera
	{
		pos.x = targetEnt->getPosX()+offset.x;
		pos.y = targetEnt->getPosY()+offset.y;
	}

	/*---------- Update velocity  --------*/
	
	if(status < 1.0f) //If camera has independent movement, find velocity based on previous position
	{
		setVelX( getPosX() - previousPos.x );
		setVelY( getPosY() - previousPos.y );
	}else //If camera does not have independent movment, copy velocity from target entity
	{
		setVelX( targetEnt->getVelX() );
		setVelY( targetEnt->getVelY() );		
	}
}

void CameraEntity::setTargetEnt(DynamicEntity* ent)
{
	targetEnt = ent;
}

void CameraEntity::setStartPos(Vector pos_)
{
	startPos=pos_;	
};