/***********************************************************************************/
/*                                                                                 */
/*                         DrawEngine Class Definition                             */
/*                                                                                 */
/***********************************************************************************/


#include "drawEngine.h"
#include "entity.h"
#include "dynamicEntity.h"

/*------------------------------------------------------------------------------------------------*/
/*------------------------------------- Constructor  ---------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/

 DrawEngine::DrawEngine(Scene* scenePtr_)
{
	scenePtr = scenePtr_;

	farClip = 200.0f;

	initGL();
	initIL();

	textureLoadScreen = LoadImage("sprites/loading.png");
	drawLoadScreen();

	textures[TEX_PLAYER][0] = LoadImage("sprites/egg_glow.png");
	textures[TEX_PLAYER][1] = LoadImage("sprites/egg.png");
	textures[TEX_BLOCK][0] = LoadImage("sprites/block_glow.png");
	textures[TEX_BLOCK][1] = LoadImage("sprites/block.png");
	textures[TEX_DYNAMIC][0] = textures[TEX_BLOCK][0];
	textures[TEX_DYNAMIC][1] = textures[TEX_BLOCK][1];
	textures[TEX_BLOOM][0] = LoadImage("sprites/bloom.png"); 
	textures[TEX_BLOOM][1] = textures[TEX_BLOOM][0]; 
	textures[TEX_MOUSE][0] = textures[TEX_BLOOM][0];
	textures[TEX_MOUSE][1] = textures[TEX_BLOOM][0]; 
	textures[TEX_SPARK][0] = LoadImage("sprites/spark_glow.png"); 
	textures[TEX_SPARK][1] = LoadImage("sprites/spark.png"); 
	textures[TEX_CLOUD][0] = LoadImage("sprites/cloud.png"); 
	textures[TEX_CLOUD][1] = textures[TEX_CLOUD][0]; 
	textures[TEX_STAR][0] = textures[TEX_BLOOM][0]; 
	textures[TEX_STAR][1] = LoadImage("sprites/star.png"); 
	textures[TEX_BEETLE][0] = LoadImage("sprites/beetle_glow.png"); 
	textures[TEX_BEETLE][1] = LoadImage("sprites/beetle.png"); 
	textures[TEX_GLIDER][0] = LoadImage("sprites/glider_glow.png"); 
	textures[TEX_GLIDER][1] = LoadImage("sprites/glider.png"); 


	mousePtr = scenePtr->getMouseEnt();
	playerPtr = scenePtr->getPlayerEnt();
	cameraPtr = scenePtr->getCameraEnt();
	borealisPtr = scenePtr->getBorealis();
}

 DrawEngine::~DrawEngine()
 {
	 glDeleteTextures(1, (GLuint*)&textureLoadScreen);
	 for(int i=0; i <= TEX_GLIDER; i++)
	 {
		glDeleteTextures(2, (GLuint*)textures[i]);
	 }
}


/*------------------------------------------------------------------------------------------------*/
/*------------------------------------ Accessor Functions ----------------------------------------*/
/*------------------------------------------------------------------------------------------------*/


bool DrawEngine::setScene(Scene* scenePtr_)
{
	scenePtr = scenePtr_;
	return 0;
}



/*------------------------------------------------------------------------------------------------*/
/*------------------------------------- Draw Functions -------------------------------------------*/
/*------------------------------------------------------------------------------------------------*/


void DrawEngine::draw()
{
	int sceneWidth, sceneHeight;

	sceneWidth = scenePtr->getSceneWidth();
	sceneHeight = scenePtr->getSceneHeight();

	glClear(GL_COLOR_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glColor3f(1.0, 1.0, 1.0);

	/*-------------Draw static objects--------------*/

	drawList(scenePtr->getStaticEntList(), scenePtr->getCameraEnt());

	/*------------------------Draw player object------------------------*/


	glPushMatrix();

		glLoadIdentity();
		glTranslatef(-cameraPtr->getPosX(), -cameraPtr->getPosY(), -20.0f);
		glTranslatef(playerPtr->getPosX(), playerPtr->getPosY(), 0);
		drawEntity(playerPtr);

	glPopMatrix();


	/*---------------Draw dynamic objects----------------*/

	drawList(scenePtr->getDynEntList(), scenePtr->getCameraEnt());


	/* --------------------- Draw Decal Objects ------------------------*/

	drawList(scenePtr->getDecalEntList(), scenePtr->getCameraEnt());
	

	/* --------------------- Draw Background Objects ------------------------*/

	drawBackdrop();


	/*----------------------- Swap Buffers -------------------------------*/

	glFlush();
	SDL_GL_SwapBuffers();
}


void DrawEngine::drawList(linkedList* list, Entity* camera)
{
	Entity* ent;
	int sceneWidth, sceneHeight;

	sceneWidth = scenePtr->getSceneWidth();
	sceneHeight = scenePtr->getSceneHeight();

	ent = list->getHead();

	while(ent != NULL)
	{
	
		glPushMatrix();

			glLoadIdentity();
			glTranslatef(-camera->getPosX(), -camera->getPosY(), -20.0f); //apply camera transformation
			glTranslatef(ent->getPosX(), ent->getPosY(), 0);

			drawEntity(ent);

			//draw object again- stage transpose x
			if( (camera->getPosX()/2) < (sceneWidth/2) )
				glTranslatef((float)-(sceneWidth*2), 0, 0);
			else
				glTranslatef((float)(sceneWidth*2), 0, 0);
			drawEntity(ent);

			//draw object again- stage transpose x+y

			if( (camera->getPosY()/2) < (sceneHeight/2) )
				glTranslatef(0, (float)-(sceneHeight*2),  0);
			else
				glTranslatef(0, (float)(sceneHeight*2),  0);
			drawEntity(ent);

			//draw object again- stage transpose y (x+y-x)

			if( (camera->getPosX()/2) < (sceneWidth/2) )
				glTranslatef((float)(sceneWidth*2), 0, 0);
			else
				glTranslatef((float)-(sceneWidth*2), 0, 0);
			drawEntity(ent);

			ent = list->getNext();

		glPopMatrix();
	}
}


void DrawEngine::drawEntity(Entity* ent)
{
		float size = ent->getSize()*2.0f;

		glRotatef((GLfloat)ent->rotate, 0, 0, 1);

		/*---------Draw glow-----------*/

		glBindTexture(GL_TEXTURE_2D, textures[ent->getTextureType()][0]);
		glColor4f( ent->getGlowColour('R'), ent->getGlowColour('G'), ent->getGlowColour('B'), ent->getGlowColour('A') );

		drawBillboard(size);

		/*-----------Draw line-------------*/

		glBindTexture(GL_TEXTURE_2D, textures[ent->getTextureType()][1]);
		glColor4f( 1.0, 1.0, 1.0, ent->getLineAlpha() );

		drawBillboard(size);

		/*------------Draw bloom-----------*/

		size = ent->getSize()*3.0f;

		glBindTexture(GL_TEXTURE_2D, textures[TEX_BLOOM][0]);
		glColor4f( ent->getGlowColour('R'), ent->getGlowColour('G'), ent->getGlowColour('B'), 0.4f * ent->getGlowColour('A') );

		drawBillboard(size);

		glRotatef(-(GLfloat)ent->rotate, 0, 0, 1);
}


void DrawEngine::drawBillboard(float size)
{
		glBegin(GL_QUADS);		
			glTexCoord2i(1,0);	glVertex3f( size, size, 0.0f );
			glTexCoord2i(0,0);	glVertex3f(-size, size, 0.0f );
			glTexCoord2i(0,1);	glVertex3f(-size, -size, 0.0f );
			glTexCoord2i(1,1);	glVertex3f( size, -size, 0.0f );
		glEnd();
}


void DrawEngine::drawBackdrop()
{
	float width, height;
	width = aspect * 8.0f;
	height = 8.0f;
	
	drawList(scenePtr->getBackgroundEntList(), scenePtr->getBackgroundCamera());

	glDisable(GL_TEXTURE_2D);

	glPushMatrix();
		glLoadIdentity();
		glBegin(GL_QUAD_STRIP);		
			glColor4f( borealisPtr->colour1.r, borealisPtr->colour1.g, borealisPtr->colour1.b, borealisPtr->colour1.a );
			glVertex3f(-width,		height, -20.0f );

			glColor4f( borealisPtr->colour1.r, borealisPtr->colour1.g, borealisPtr->colour1.b, 0.0f );
			glVertex3f(-width,		-height+3.0f, -20.0f );

			glColor4f( borealisPtr->colour2.r, borealisPtr->colour2.g, borealisPtr->colour2.b, borealisPtr->colour2.a );
			glVertex3f(-width*0.5f,	height, -20.0f );

			glColor4f( borealisPtr->colour2.r, borealisPtr->colour2.g, borealisPtr->colour2.b, 0.0f );
			glVertex3f(-width*0.5f,	-height+7.5f, -20.0f );

			glColor4f( borealisPtr->colour3.r, borealisPtr->colour3.g, borealisPtr->colour3.b, borealisPtr->colour3.a );
			glVertex3f( 0,			height, -20.0f );

			glColor4f( borealisPtr->colour3.r, borealisPtr->colour3.g, borealisPtr->colour3.b, 0.0f );
			glVertex3f( 0,			-height+9.0f, -20.0f );

			glColor4f( borealisPtr->colour4.r, borealisPtr->colour4.g, borealisPtr->colour4.b, borealisPtr->colour4.a );
			glVertex3f( width*0.5f,	height, -20.0f );

			glColor4f( borealisPtr->colour4.r, borealisPtr->colour4.g, borealisPtr->colour4.b, 0.0f );
			glVertex3f(width*0.5f,	-height+7.5f, -20.0f );

			glColor4f( borealisPtr->colour5.r, borealisPtr->colour5.g, borealisPtr->colour5.b, borealisPtr->colour5.a );
			glVertex3f(width,		height, -20.0f );

			glColor4f( borealisPtr->colour5.r, borealisPtr->colour5.g, borealisPtr->colour5.b, 0.0f );
			glVertex3f(width,		-height+3.0f, -20.0f );
		glEnd();
	glPopMatrix();

	glEnable(GL_TEXTURE_2D);
}

void DrawEngine::drawLoadScreen()
{
	float width, height;
	width = aspect * 8.0f;
	height = 8.0f;
	float size = 5.0f;

	glClear(GL_COLOR_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glColor3f(1.0, 1.0, 1.0);
	glEnable(GL_TEXTURE_2D);


	glBindTexture(GL_TEXTURE_2D, textureLoadScreen);
	glPushMatrix();
	glLoadIdentity();
		glTranslatef(0, 0, -20.0f);
		glBegin(GL_QUADS);		
			glTexCoord2i(1,1);	glVertex3f( size, size, 0.0f );
			glTexCoord2i(0,1);	glVertex3f(-size, size, 0.0f );
			glTexCoord2i(0,0);	glVertex3f(-size, -size, 0.0f );
			glTexCoord2i(1,0);	glVertex3f( size, -size, 0.0f );
		glEnd();
	glPopMatrix();

	glDisable(GL_TEXTURE_2D);

	glFlush();
	SDL_GL_SwapBuffers();

}



/*------------------------------------------------------------------------------------------------*/
/*------------------------------ Initialization Functions ----------------------------------------*/
/*------------------------------------------------------------------------------------------------*/

GLvoid DrawEngine::initGL()
{
	setupProjection();
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_DST_ALPHA);
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable(GL_PERSPECTIVE_CORRECTION_HINT);
}


ILvoid DrawEngine::initIL()
{
	ilInit();
	ilutRenderer(ILUT_OPENGL); 
}


GLvoid DrawEngine::setupProjection()
{
	glViewport(0, 0, scenePtr->getWindowWidth(), scenePtr->getWindowHeight());
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	aspect = (float)(scenePtr->getWindowWidth())/(scenePtr->getWindowHeight());
	glOrtho( (-8 * aspect), (8 * aspect), -8, 8, 0.1, 100.0 );

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

}


/***********************************************************************************/
/*                                                                                 */
/*                LoadImage Function Provided by James Whitworth                   */
/*                                                                                 */
/***********************************************************************************/



int DrawEngine::LoadImage (char* fileName)
{
	ILuint imageID;
	int TexID;

	ilGenImages(1, &imageID);

	ilBindImage(imageID);

	ilLoadImage((const wchar_t*)fileName);
	iluFlipImage();
	ilConvertImage(IL_RGBA, IL_UNSIGNED_BYTE);

	ILenum Error = ilGetError();

	TexID = ilutGLBindTexImage();

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); 
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); 
	glTexImage2D(GL_TEXTURE_2D, 0, ilGetInteger(IL_IMAGE_BPP), ilGetInteger(IL_IMAGE_WIDTH), ilGetInteger(IL_IMAGE_HEIGHT), 
	0, ilGetInteger(IL_IMAGE_FORMAT), GL_UNSIGNED_BYTE, ilGetData());

	ilDeleteImages(1, (ILuint*)&imageID);

	return TexID;
}
