/***********************************************************************************/
/*                                                                                 */
/*                         Entity Class Declaration                                */
/*                                                                                 */
/***********************************************************************************/

/*
The entity class holds all the attributes needed to represent a single object in the game.
This includes attributes for position, size, colour, age and lifespan.
It also includes flags to indicate what texture should be used when the entity is drawn, 
and what type of collisions the entity should produce.
*/

#ifndef ENTITY_H
#define ENTITY_H
#include "vector.h"


/*----------------------------------------------------------*/
/*-------------------- Custom datatypes --------------------*/
/*----------------------------------------------------------*/

//Flags representing collision types.
enum 
{
	CT_PLAYER,
	CT_STATIC,
	CT_DYNAMIC, 
	CT_NOHIT,
	CT_MOUSE,
	CT_SPARK
};


//Flags representing texture types.
enum 
{
	TEX_PLAYER,
	TEX_BLOCK,
	TEX_DYNAMIC, 
	TEX_BLOOM,
	TEX_MOUSE,
	TEX_SPARK,
	TEX_CLOUD,
	TEX_STAR,
	TEX_BEETLE,
	TEX_GLIDER
};

//Structure for storing colour values.
struct colour
{
	float r;
	float g;
	float b;
	float a;
};



/*-----------------------------------------------------------*/
/*-------------------- Class Declaration --------------------*/
/*-----------------------------------------------------------*/

class Entity
{
public:

	/*-------Constructor and Destructor--------*/

	Entity(float posX = 0, float posY = 0);
	~Entity();


	/*----------Get Functions-----------*/

	Vector getPosition(void);
	float getPosX(void);
	float getPosY(void);
	int getCollideType(void);
	int getTextureType(void);
	float getSize(void);
	float getLife(void);
	int getAge(void);
	float getLineAlpha();
	float getGlowColour(char);
	float getStatus(void);
	bool getXPermit(void);


	/*---------Set Functions-----------*/

	void setPosition(Vector);
	void setPosX(float x_);
	void setPosY(float y_);
	void setXPermit(bool);
	void setSize(float size_);
	void setLife(float life_);
	virtual void setStatus(float status_);
	void setLineAlpha(float a);
	void setGlowColour(float r, float g, float b, float a);
	void setGlowColour(colour c);
	void setDefaultGlowColour(colour c);
	void setGlowAlpha(float a);
	void setTextureType(int);
	colour shiftColour(colour base, colour target, float shift);

	void resetGlowColour();

	void incrPosX(float x_);
	void incrPosY(float y_);


	/*------------Entity Update Functions------------*/

	virtual void updateEntity();
	virtual void collideResponse(Vector, Entity*);


	double rotate;
protected:

	/*-------------Attributes-----------*/

	//Position vector.
	Vector pos;	

	//Collide-type and Texture-type flags.
	int collideType;
	int textureType;

	//Entity age.
	long int age;

	//Entity size.
	float size;

	//Entity status (used to define different stages of behaviour).
	float status;

	//'Life' remaining.
	float life;

	//Collision indicators.
	int hit;
	bool xpermit;

	//Colour values.
	colour glowColour;
	colour glowColourDefault;
	colour targetColour;

	float lineAlpha;
	float lineAlphaDefault;

};


#endif