/***********************************************************************************/
/*                                                                                 */
/*                       Player Entity Class Definition                            */
/*                                                                                 */
/***********************************************************************************/

#include <iostream>
#include "playerEntity.h"
#include "math.h"


/*----------------------------------------------------*/
/*------------------- Constructor --------------------*/
/*----------------------------------------------------*/

PlayerEntity::PlayerEntity(float x, float y) : DynamicEntity(x, y)
{
	collideType = CT_PLAYER;
	textureType = TEX_PLAYER;

	setGlowColour(1.0f, 1.0f, 1.0f, 0.15f);

	glowColourDefault.r = targetColour.r = glowColour.r;
	glowColourDefault.g = targetColour.g = glowColour.g;
	glowColourDefault.b = targetColour.b = glowColour.b;
	glowColourDefault.a = targetColour.a = glowColour.a;

	topSpeedColour.r=0.1f;
	topSpeedColour.g=1.5f;
	topSpeedColour.b=1.5f;
	topSpeedColour.a=1.2f;

	setLineAlpha(1.0f);
	rotate = 180;
	speed = 0;
	sparks = 0;

	status=1.0f;
	jumpStatus=0;
	bigJumpCount=0.0f;
	mouseEnt=NULL;
	mouseAngle = 0;
	previousMouseAngle = 0;
	mouseAngleVelocity = 0;

	age=0;
}




/*---------------------------------------------------------*/
/*-------------------- Update Function --------------------*/
/*---------------------------------------------------------*/


void PlayerEntity::updateEntity()
{
	/*------------- Position and Velocity Updates -------------*/


	//Change velocity if player has collided with a dynamic object.

	if(dynamicHit==1) 
	{
		setVelX(getHitVelX());
		setVelY(getHitVelY());
		dynamicHit = 0;
	}

	//Interpret mouse behaviour.

	previousMouseAngle = mouseAngle;
	mouseAngle = getMouseAngle();
	mouseAngleVelocity = mouseAngle - previousMouseAngle;

	if(mouseAngleVelocity > 3.14159f)
		mouseAngleVelocity -= 6.28318f;
	else if(mouseAngleVelocity < -3.14159f)
		mouseAngleVelocity += 6.28318f;
	sparks = 0;

	//Update player position according to the current velocity.

	setPosX( getPosX() + getVelX() );
	setPosY( getPosY() + getVelY() );

	if(bigJumpCount==1 && status < 4.0f)
		setStatus(4);

	//Record player speed.

	speed = vel.getMagnitude();

	//Change velocity based on player status.

	if(getStatus()==1)  // Standard velocity changes (status 1).
	{
		setVelY( 0.005f*cos((float)age/75.0f));		
	}
	else if(getStatus() > 1 && getStatus() < 2.1f) // Standard velocity changes (status 2).
	{
		if(getStatus() < 2.0f)
			status = status + 0.1f;
		else
			setStatus(2.0f);

		//Apply velocity based on mouse behaviour.
		if(mouseEnt->getPlayerGrabStatus())
		{
			if(getMouseDistance()>1.0)
				incrVelX(-0.01f*mouseAngleVelocity);
		}

		//Gravity.
		setVelY( (float)(getVelY() - 0.02) );

		//Velocity damping.
		setVelX( getVelX() - (getVelX()/100) );
		setVelY( getVelY() - (getVelY()/100) );

		//Rotation update.
		rotate-=getVelX()*50;
		if(rotate>360)
			rotate=(rotate-360);

	}else if(getStatus() > 2.1f && getStatus() < 3.1f  ) // Standard velocity changes (status 3).
	{
		//Apply velocity based on mouse behaviour.
		if(mouseEnt->getPlayerGrabStatus())
		{
			if(getMouseDistance()>1.0)
				incrVelX(-0.015f*mouseAngleVelocity);
		}

		//Apply gravity.
		setVelY( (float)(getVelY() - 0.02) );

		//Velocity damping.
		if(jumpStatus==0)
			setVelX( getVelX() - (getVelX()/200) );

		setVelY( getVelY() - (getVelY()/200) );

		if(mouseEnt->getPlayerJumpStatus())
		{
			mouseEnt->setPlayerJumpStatus(0);
			jump();
		}
	}
	else if(getStatus() > 3.9f) // Standard velocity changes (status 4).
	{
		if(speed!=0)
			rotate =  ((360/6.263185)*((atan2( getVelY(), getVelX() ))+1.57079)) ;

		if(mouseEnt->getPlayerGrabStatus())
		{
			setVelX( getVelX() + 0.01f*(mouseEnt->getUnwrappedPosX() - getPosX()) );
			setVelY( getVelY() + 0.01f*(mouseEnt->getUnwrappedPosY() - getPosY()) );
		}

		if(vel.getMagnitude() > 1.0f)
			setVelX( 0.95f*getVelX() );
			setVelY( 0.95f*getVelY() );
	}


	if(getStatus() <= 3.0f)	//Apply sparks to fast-moving player.
	{
		if(speed > 0.5)
		{
			if(rand()%(3) == 0)
			sparks =1;

			if(speed > 0.7)
			{
				sparks +=2;
			}
		}
	}

	age++;


	/*-------------------- Colour Updates ---------------------*/

	if(hit>0)
		hit--; 

	//Use player speed to interpolate between default colour and moving colour.
	float t=speed*1.2f;
	if(t>1.0)
		t=1.0;

	targetColour.r = (1.0f - t)*glowColourDefault.r + t*topSpeedColour.r;
	targetColour.g = (1.0f - t)*glowColourDefault.g + t*topSpeedColour.g;
	targetColour.b = (1.0f - t)*glowColourDefault.b + t*topSpeedColour.b;
	targetColour.a = (1.0f - t)*glowColourDefault.a + t*topSpeedColour.a;


	//Actual glow colour changes incrementally to match target colour.

	if(glowColour.r != targetColour.r)
	{
		glowColour.r += 0.05f*(targetColour.r - glowColour.r);
	}
	if(glowColour.g != targetColour.g)
	{
		glowColour.g += 0.05f*(targetColour.g - glowColour.g);
	}

	if(glowColour.b != targetColour.b)
	{
		glowColour.b += 0.05f*(targetColour.b - glowColour.b);
	}

	if(glowColour.a != targetColour.a)
	{
		glowColour.a += 0.05f*(targetColour.a - glowColour.a);
	}

	//Increment age.
	age++;
}




/*---------------------------------------------------------------*/
/*----------------- Collision Response Function -----------------*/
/*---------------------------------------------------------------*/


void PlayerEntity::collideResponse(Vector proj, DynamicEntity* ent)
{

	if(ent!=NULL && (proj.x != 0 || proj.y != 0))
	{
		
		if(ent->getCollideType() == CT_STATIC) //colliding with a static entity.
		{

			if(getStatus() < 4.0f)
			{
				if(hit==0)
					setGlowAlpha(glowColourDefault.a+0.5f);

				hit=7;

				if(proj.x != 0)		//collision on x
				{
						//project entity, change x velocity.
					if(ent->getXPermit() == 1)
					{
						setVelX(0);
						incrPosX(-proj.x);
						staticHit = 0;
					}
				}
				else if(proj.y != 0)	//collision on y
				{
					if( getVelY() < -0.3f)
					{
						sparks = (int)(-10.0f*getVelY())-2;
						if(sparks < 1)
							sparks=1;
					}

					if( getVelY() < -0.7 && getStatus() < 3.0f )
					{
						setStatus(3.0f);
					}

					jumpStatus=0;
					if(bigJumpCount>0)
						bigJumpCount = bigJumpCount-0.1f;

					//project entity, change y velocity.
					incrPosY(-(proj.y));
					setVelY( (float)-0.1*(getVelY()) );
				}

			}else
			{
				setGlowColour(1.0f, 1.0f, 1.0f, 1.0f);
				sparks=10;
			}
		}
		else if(ent->getCollideType() == CT_DYNAMIC) //colliding with a dynamic entity.
		{
			if(hit==0)
				setGlowAlpha(glowColourDefault.a+0.5f);

			hit=7;

			dynamicHit=1; //next time update function is called, vel will be swapped with hitvel
			if(proj.x != 0) //collision on x
			{
				incrPosX(-(proj.x));

				setHitVelX( ent->getVelX( ));
				setHitVelY( getVelY( ));
			}
			else if(proj.y != 0)
			{
				incrPosY(-(proj.y));

				setHitVelY( (float)-0.1*(getVelY())+ent->getVelY() );
				setHitVelX( getVelX( ));
			}
		}

		else if(ent->getCollideType() == CT_MOUSE)
		{
			if(mouseEnt->getClickStatus() > 0)
			{
				setGlowAlpha(glowColourDefault.a+0.5f);

				if(status == 1.0f)
					setStatus(1.1f);
			}
		}
	}
}



/*-------------------------------------------------------------*/
/*----------------- Status Change Functions --------------------*/
/*-------------------------------------------------------------*/


void PlayerEntity::setStatus(float status_)
{
	status = status_;

	if(status == 3.0)
	{
		glowColourDefault.r = 0.0f;
		glowColourDefault.g = 1.0f;
		glowColourDefault.b = 0.0f;
		glowColourDefault.a = 0.2f;
		topSpeedColour.r = 1.0f;
		topSpeedColour.g = 0.7f;
		topSpeedColour.b = 0.0f;
		topSpeedColour.a = 1.0f;
		setTextureType(TEX_BEETLE);

		setGlowColour(1.0, 1.0, 1.0, 1.0);
		sparks = 30;
		rotate = 180.0f;
		setVelX(0.0f);
	}
	else if(status == 4.0)
	{
		glowColourDefault.r = 1.0f;
		glowColourDefault.g = 0.0f;
		glowColourDefault.b = 0.0f;
		glowColourDefault.a = 0.2f;
		topSpeedColour.r = 1.0f;
		topSpeedColour.g = 0.0f;
		topSpeedColour.b = 0.0f;
		topSpeedColour.a = 1.0f;
		setTextureType(TEX_GLIDER);

		setGlowColour(1.0, 1.0, 1.0, 1.0);
		sparks = 30;
		rotate = ((360/6.263185)*getMouseAngle())-1.57079;
	}
}

void PlayerEntity::jump(void)
{

	if(jumpStatus==0)
	{
		setPosY(getPosY()+0.1f);
		setVelY( 0.5f*fabs(getVelX()) + 0.15f );

		setGlowColour(1.0f, 1.0f, 1.0f, 1.0f);

		if( fabs(getVelX()) > 0.65)
		{
			setGlowColour(1.0f, 0.5f, 0.5f, 1.0f);
		}

		if( fabs(getVelX()) > 0.9)
			bigJumpCount=1;

		sparks+=5;

		jumpStatus=-1;
	}
}




/*-------------------------------------------------------------*/
/*--------------------- Accessor Functions --------------------*/
/*-------------------------------------------------------------*/


float PlayerEntity::getMouseDistance(void)
{
	if(mouseEnt != NULL)
	{
		return sqrt( ((mouseEnt->getUnwrappedPosX()-getPosX()) * (mouseEnt->getUnwrappedPosX()-getPosX())) + 
					 ((mouseEnt->getUnwrappedPosY()-getPosY()) * (mouseEnt->getUnwrappedPosY()-getPosY())) );
	}
	else
		return 0.0f;
}

float PlayerEntity::getMouseAngle(void)
{
	if(mouseEnt != NULL)
	{
		if( (mouseEnt->getUnwrappedPosX()-getPosX()) != 0 )
			return atan2( (mouseEnt->getUnwrappedPosY()-getPosY()), (mouseEnt->getUnwrappedPosX()-getPosX()) );
	}


	return previousMouseAngle;
}

int PlayerEntity::getSparkStatus(void)
{
	return sparks;	
}

void PlayerEntity::setMouseEnt(MouseEntity* ent)
{
	mouseEnt = ent;
}
