/***********************************************************************************/
/*                                                                                 */
/*                          Scene Class Definition                                 */
/*                                                                                 */
/***********************************************************************************/

#include "scene.h"
#include "math.h"
#include <iostream>

using namespace std;

 
/*------------------------------------------------------------------------------------------------*/
/*-------------------------------- Constructor & Destructor --------------------------------------*/
/*------------------------------------------------------------------------------------------------*/

Scene::Scene(int scenePreset, int windowWidth_, int windowHeight_)
{

	windowWidth = windowWidth_;
	windowHeight = windowHeight_;

	sceneWidth=70;
	sceneHeight=50;
	stage = 0;
	starCount = 0;
 
	/*----------Initialize scene array----------*/

	sceneArray = (EntityPtr**) calloc(sceneWidth, sizeof(EntityPtr*));

	for(int i=0; i<sceneWidth; i++)
	{
		sceneArray[i] = (EntityPtr*) calloc(sceneWidth, (sizeof(EntityPtr)) );

		for(int j=0; j<sceneHeight; j++)
		{
			sceneArray[i][j] = NULL;
		}
	}

	/*----------------Initialize player object----------------*/

	playerEnt.setPosX( (float)(sceneWidth) );
	playerEnt.setPosY( (float)(sceneHeight*2) - 2.0f );
	addDynEnt((Entity*)&playerEnt);


	/*-------------------Initialize cameras--------------------*/

	cameraEnt.setPosX(playerEnt.getPosX()); 
	cameraEnt.setPosY(playerEnt.getPosY()-3.0f);
	cameraEnt.setStartPos( cameraEnt.getPosition() );
	cameraEnt.setTargetEnt(&playerEnt);

	backgroundCamera.setPosX(cameraEnt.getPosX());
	backgroundCamera.setPosY(cameraEnt.getPosY());


	/*-------------------Initialize mouse---------------------*/

	mouseEnt = new MouseEntity(0, 0);
	mouseEnt->setPosX(playerEnt.getPosX());
	mouseEnt->setPosY(playerEnt.getPosY());
	mouseEnt->setSize((float)0.3);
	mouseEnt->setCameraEnt(&cameraEnt);
	playerEnt.setMouseEnt(mouseEnt);


	/*------------- Initialize scene colours ---------------*/

	sceneColour1.r = 0.6f;
	sceneColour1.g = 0.5f;
	sceneColour1.b = 1.0f;
	sceneColour1.a = 0.0f;

	sceneColour2.r = 0.0f;
	sceneColour2.g = 1.0f;
	sceneColour2.b = 1.0f;
	sceneColour2.a = 0.0f;

	sceneBorealis.setDefaultColours(sceneColour1, sceneColour2);


	/*---------------- Add static objects -----------------*/

	Entity *newEntity;

	//top ramp
	for(int i=4; i<sceneWidth-3; i++)
		addBlock(i, sceneHeight-5);

	//sidewalls top
	for(int i=sceneHeight-2; i>sceneHeight-9; i--)
		addBlock(1, i);

	for(int i=sceneHeight-2; i>sceneHeight-9; i--)
		addBlock(sceneWidth-1, i);

	//floor 2
	for(int i=2; i<8; i++)
		addBlock(i, sceneHeight-8);

	//floor 2 right
	for(int i=sceneWidth-2; i>sceneWidth-8; i--)
		addBlock(i, sceneHeight-8);

	//floor 3
	for(int i=4; i<13; i++)
		addBlock(i, sceneHeight-11);

	//floor 3 right
	for(int i=sceneWidth-4; i>sceneWidth-13; i--)
		addBlock(i, sceneHeight-11);

	//sidwalls low
	for(int i=sceneHeight-6; i>sceneHeight-11; i--)
		addBlock(12, i);

	for(int i=sceneHeight-6; i>sceneHeight-11; i--)
		addBlock(sceneWidth-12, i);

	//ground floor
	for(int i=0; i<sceneWidth; i++)
		addBlock(i, +2);

	removableBlock1=NULL;
	removableBlock2=NULL;



	/*--------------- Add decal objects ---------------*/

	//clouds
	float t;
	for(int i=0; i<70; i++)
	{
		t = ((float)rand()/RAND_MAX);
		colour temp;

		temp.r = (t*sceneColour1.r) + ((1.0f-t)*sceneColour2.r);
		temp.g = (t*sceneColour1.g) + ((1.0f-t)*sceneColour2.g);
		temp.b = (t*sceneColour1.b) + ((1.0f-t)*sceneColour2.b);
		temp.a = 0.025f;

		newEntity = new CloudEntity( (float)(rand()%(sceneWidth*2)), (float)(rand()%(sceneHeight*2)) );

		newEntity->setDefaultGlowColour( temp );
		newEntity->setGlowColour( temp );

		addDecalEnt(newEntity);
	}

	//mouse trail
	glowLineLength=20;
	for(int i=0; i<glowLineLength; i++)
	{
		newEntity = new BloomEntity( playerEnt.getPosX(), playerEnt.getPosY() );
		newEntity->setStatus(BLOOM_FIXED);
		newEntity->setSize(0.5);
		newEntity->setGlowColour(playerEnt.getGlowColour('R'), playerEnt.getGlowColour('G'), playerEnt.getGlowColour('B'), 0.0 );

		glowLine[i] = newEntity;
		decalEntList.addTail(newEntity);
	}

}


Scene::~Scene()
{
	//delete static and dynamic objects

	dynEntList.~linkedList();
	staticEntList.~linkedList();
	decalEntList.~linkedList();

	//free memory reserved for sceneArray
	for(int i=sceneWidth-1; i>-1; i--)
	{
		free(sceneArray[i]); //free the row
	}
	free(sceneArray);
}


/*------------------------------------------------------------------------------------------------*/
/*------------------------------------ Accessor Functions ----------------------------------------*/
/*------------------------------------------------------------------------------------------------*/



/*--------- Access important entities ---------*/

DynamicEntity* Scene::getPlayerEnt()
{
	return &playerEnt; //Return pointer to player entity.
}

CameraEntity* Scene::getCameraEnt()
{
	return &cameraEnt; //Return pointer to player entity.
}

DynamicEntity* Scene::getBackgroundCamera()
{
	return &backgroundCamera; //Return pointer to player entity.
}

MouseEntity* Scene::getMouseEnt()
{
	return mouseEnt; //Return pointer to player entity.
}

Borealis* Scene::getBorealis()
{
	return &sceneBorealis;
}


/*---------- Access Entity Lists -------------*/


linkedList* Scene::getDynEntList() 
{
	//Return pointer to the list of dynamic scene entities.
	//(should be redundant with dynamic entity access functions).
	return &dynEntList;
}

linkedList* Scene::getStaticEntList()
{
	return &staticEntList; //Return pointer to the list of static scene entities.
}

linkedList* Scene::getDecalEntList()
{
	return &decalEntList; //Return pointer to the list of static scene entities.
}


linkedList* Scene::getBackgroundEntList()
{
	return &backgroundEntList;
}

Entity* Scene::getFirstDynEnt()
{
	return dynEntList.getHead(); //Return pointer to first dynamic entity
}
Entity* Scene::getNextDynEnt()
{
	return dynEntList.getNext(); //Return pointer to next dynamic entity
}

Entity* Scene::getFirstStaticEnt()
{
	return staticEntList.getHead(); //Return pointer to first static entity
}
Entity* Scene::getNextStaticEnt()
{	
	return staticEntList.getNext(); //Return pointer to next static entity
}

Entity* Scene::getFirstDecal()
{
	return decalEntList.getHead();	//Return pointer to first decal entity
}
Entity* Scene::getNextDecal()
{
	return decalEntList.getNext();	//Return pointer to next decal entity
}

Entity* Scene::getFirstBackgroundEnt()
{
	return backgroundEntList.getHead();	//Return pointer to first decal entity
}
Entity* Scene::getNextBackgroundEnt()
{
	return backgroundEntList.getNext();	//Return pointer to next decal entity
}


/*-----------Access scene array------------*/

Entity*	Scene::getSceneArrayElement(int x, int y)
{
	if(x > sceneWidth-1)
		x = x-sceneWidth;
	else if(x < 0)
		x = x+sceneWidth;

	if(y > sceneHeight-1)
		y = y-sceneHeight;
	else if(y < 0)
		y = y+sceneHeight;

	return sceneArray[x][y];
}


/*--------------- Access scene dimensions -----------------*/

int Scene::getSceneWidth()
{
	return sceneWidth;
}

int Scene::getSceneHeight()
{
	return sceneHeight;
}


/*--------------- Access window dimensions ----------------*/


void Scene::setWindowWidth(int windowWidth_)
{
	windowWidth = windowWidth_;
}

int Scene::getWindowWidth()
{
	return windowWidth;
}

void Scene::setWindowHeight(int windowHeight_)
{
	windowHeight = windowHeight_;
}

int Scene::getWindowHeight()
{
	return windowHeight;
}



/*------------------------------------------------------------------------------------------------*/
/*-------------------------------- Adding & Removing Scene Entities ------------------------------*/
/*------------------------------------------------------------------------------------------------*/

int Scene::addDynEnt(Entity* ent)
{
	dynEntList.addTail(ent);
	return 0;
}

int Scene::addStaticEnt(Entity* ent) 
{
	//Register a new static entity in the scene, add it to staticEntList and sceneArray.

	//Add static entity to staticEntList.

	staticEntList.addTail(ent); 

	//Convert object coordinates to array indices.

	int x,y;
	x= (int)( ((ent->getPosX())/2) ); 	
	y= (int)( ((ent->getPosY())/2) );

	//Check that the indices are safe to use.

	if( (x < sceneWidth) && (x >= 0) && (y < sceneHeight) && (y > -1) )
	{
		//Add entity to scene array.
		sceneArray[x][y] = ent;
	}

	return 0;
}


int Scene::addDecalEnt(Entity* ent)
{
	decalEntList.addTail(ent); 
	return 0;
}

bool Scene::deleteDynEnt(Entity* ent)
{
	dynEntList.deleteNode(ent);
}

int Scene::addBackgroundEnt(Entity* ent)
{
	backgroundEntList.addTail(ent); 
	return 0;
}

Entity* Scene::addBlock(int x, int y)
{
	Entity* adjacent1, *adjacent2;

		//check adjacent squares left

		adjacent1 = getSceneArrayElement(x-1, y);
		adjacent2 = getSceneArrayElement(x-2, y);

		if(adjacent1 != NULL && adjacent2 != NULL)
		{
			adjacent1->setXPermit(0);
		}

		//check adjacent squares right

		adjacent1 = getSceneArrayElement(x+1, y);
		adjacent2 = getSceneArrayElement(x+2, y);

		if(adjacent1 != NULL && adjacent2 != NULL)
		{
			adjacent1->setXPermit(0);
		}



	Entity* newEntity = new Entity( x*2.0f, y*2.0f );
	addStaticEnt(newEntity);

	adjacent1 = getSceneArrayElement(x-1, y);
	adjacent2 = getSceneArrayElement(x+1, y);

	if(adjacent1 != NULL && adjacent2 != NULL)
	{
		newEntity->setXPermit(0);
	}


	return newEntity;

}
	

void Scene::deleteStaticEnt(Entity* ent)
{
	int x = (int)(ent->getPosX()/2);
	int y = (int)(ent->getPosY()/2);

	sceneArray[x][y]=NULL;

	staticEntList.deleteNode(ent);
}

void Scene::addTrail(DynamicEntity* ent)
{
	if(ent != NULL && (ent->getCollideType() == CT_DYNAMIC || ent->getCollideType() == CT_PLAYER ))
	{
		Entity* bloomEnt;
		bloomEnt = new BloomEntity(	ent->getPosX(), ent->getPosY()	);

		bloomEnt->setGlowColour(ent->getGlowColour('R'), 
								ent->getGlowColour('G'), 
								ent->getGlowColour('B'), 
								ent->getGlowColour('A')*(fabs(ent->getVelX())+fabs(ent->getVelY())/2)*0.5f );

		bloomEnt->setSize(ent->getSize());
		addDecalEnt(bloomEnt);

	}
}


/*------------------------------------------------------------------------------------------------*/
/*------------------------------- Scene Update Functions -----------------------------------------*/
/*------------------------------------------------------------------------------------------------*/

void Scene::updateScene()
{
	/*-------------Update scene status.---------------*/

	if(stage < 1 && cameraEnt.getStatus() > 0.9f)
		setStage(1);
	else if(stage < 2 && playerEnt.getStatus() > 2.9f)
		setStage(2);
	else if(stage < 4 && playerEnt.getStatus() > 3.9f)
		setStage(4);

	//Update mouse position.
	performCollisions(mouseEnt);
	mouseEnt->updateEntity();

	/*--------------Update static scene entities.--------------*/

	Entity* ent;
	ent = staticEntList.getHead();
	while(ent != NULL)
	{
		ent->updateEntity();
		wrapEntPosition(ent);

		if(ent->getLife() <= 0)
		{
			deleteStaticEnt(ent);
		}

		ent = staticEntList.getNext();
	}


	if(removableBlock1 != NULL && removableBlock2 != NULL)
	{
		if(	(playerEnt.getPosX() > removableBlock1->getPosX()-0.3 && playerEnt.getPosX() < removableBlock1->getPosX()+0.3)
		||	(playerEnt.getPosX() > removableBlock2->getPosX()-0.3 && playerEnt.getPosX() < removableBlock2->getPosX()+0.3) )
		{
			setStage(3);
			DynamicEntity* newEntity;

			for(int i=0; i<30; i++)
			{
				newEntity = new SparkEntity( removableBlock1->getPosX(), removableBlock1->getPosY() );
				newEntity->setVelX(newEntity->getVelX() * 5.0f);
				addDynEnt((Entity*)newEntity);
			}

			for(int i=0; i<30; i++)
			{
				newEntity = new SparkEntity( removableBlock2->getPosX(), removableBlock2->getPosY() );
				newEntity->setVelX(newEntity->getVelX() * 5.0f);
				addDynEnt((Entity*)newEntity);
			}

			deleteStaticEnt(removableBlock1);
			deleteStaticEnt(removableBlock2);
			removableBlock1=NULL;
			removableBlock2=NULL;
		}
	}


	/*---------------Update dynamic scene entities.------------------*/

	ent = dynEntList.getHead();
	while(ent != NULL)
	{
		if(ent->getCollideType() != CT_PLAYER)
		{
			ent->updateEntity();
			wrapEntPosition(ent);
			performCollisions((DynamicEntity*)ent);

			//remove dead entities

			if(ent->getLife() <= 0)
			{
				dynEntList.deleteNode(ent);
			}
		}
		ent = dynEntList.getNext();
	}

	/*-------------------Update decal entities.------------------*/

	ent = decalEntList.getHead();
	while(ent != NULL)
	{
		ent->updateEntity(); 


		if(ent->getLife() <= 0)
		{
			decalEntList.deleteNode(ent);
		}

		if(ent->getTextureType() == TEX_CLOUD) //apply fake parallax to clouds
		{
			ent->setPosX( ent->getPosX() - 0.5f*((ent->getSize()-4.0f)* cameraEnt.getVelX()) );
			ent->setPosY( ent->getPosY() - 0.5f*((ent->getSize()-4.0f)* cameraEnt.getVelY()) );
		}

		wrapEntPosition(ent);

		ent = decalEntList.getNext();
	}


	/*--------------Update background entities-----------*/

	if(stage > 0)
	{
		if(starCount < 150)
		{
			Entity* newEntity = new StarEntity( (float)(rand()%(sceneWidth*2)), (float)(rand()%(sceneHeight*2)) );
			addBackgroundEnt(newEntity);
			starCount++;
		}
		
		ent = backgroundEntList.getHead();
		while(ent != NULL)
		{
			ent->updateEntity(); 
			ent = backgroundEntList.getNext();
		}

		sceneBorealis.update();
	}

	/*-------------Update effects--------------*/

	if(playerEnt.getSparkStatus())
	{
		DynamicEntity* newEntity;

		for(int i=0; i<playerEnt.getSparkStatus(); i++)
		{
			newEntity = new SparkEntity( playerEnt.getPosX(), playerEnt.getPosY() );
			newEntity->setVelX( newEntity->getVelX()*((float)playerEnt.getSparkStatus()/3.0f) );
			addDynEnt((Entity*)newEntity);
		}
	}

	for(int i=0; i<glowLineLength; i++)
	{
		float t = (float)i/glowLineLength;
		ent = glowLine[i];
		ent->setPosX( (t * playerEnt.getPosX()) + ((1.0f-t) * mouseEnt->getUnwrappedPosX() ) );
		ent->setPosY( (t * playerEnt.getPosY()) + ((1.0f-t) * mouseEnt->getUnwrappedPosY() ) );



		if(mouseEnt->getPlayerGrabStatus()>0)
		{
			ent->setGlowColour( playerEnt.getGlowColour('R'), 
								playerEnt.getGlowColour('G'), 
								playerEnt.getGlowColour('B'), 
								0.01f*playerEnt.getMouseDistance() );
		}
		else if (mouseEnt->getPlayerGrabStatus()==0)
			ent->setGlowAlpha(0.0);
	}

	//Update player entity.
	playerEnt.updateEntity(); //update position from last frame
	wrapEntPosition(&playerEnt); //wrap entity if needed
	performCollisions(&playerEnt); //check for collisions
	addTrail((DynamicEntity*)&playerEnt);

	//Update camera position.
	Vector previousCameraPos = cameraEnt.getPosition();

	backgroundCamera.setVelX(0.05f*cameraEnt.getVelX());
	backgroundCamera.setVelY(0.05f*cameraEnt.getVelY());

	cameraEnt.updateEntity();
	backgroundCamera.updateEntity();
	wrapEntPosition(&backgroundCamera);
}


void Scene::wrapEntPosition(Entity* ent)
{
	if(ent->getPosX() < 0)
		ent->incrPosX( (float)sceneWidth*2 );

	else if(ent->getPosX() > sceneWidth*2)
		ent->incrPosX( (float) -sceneWidth*2 );

	if(ent->getPosY() < 0)
		ent->incrPosY( (float)sceneHeight*2 );

	else if(ent->getPosY() > sceneHeight*2)
		ent->incrPosY(  (float)-sceneHeight*2 );
}


void Scene::setStage(int set)
{
	stage = set;

	if(stage == 1)
	{
		sceneColour1.r = 0.6f;
		sceneColour1.g = 0.5f;
		sceneColour1.b = 1.0f;
		sceneColour1.a = 0.5f;
		sceneColour2.r = 0.0f;
		sceneColour2.g = 1.0f;
		sceneColour2.b = 1.0f;
		sceneColour2.a = 0.5f;
		sceneBorealis.setDefaultColours(sceneColour1, sceneColour2);
	}

	else if(stage == 2)
	{
		float posLeft, posRight, temp;
		posLeft = playerEnt.getPosX()-30.0f;
		posRight = playerEnt.getPosX()+30.0f;

		if(posLeft < 0)
			posLeft=posLeft+(sceneWidth*2.0f);
		if(posRight > (sceneWidth*2.0f-1.0f))
			posRight=posRight-(sceneWidth*2.0f-1.0f);

		if(posRight < posLeft)
		{
			temp=posLeft;
			posLeft=posRight;
			posRight=temp;
		}

		removableBlock1 = addBlock((int)(posLeft/2), (int)(playerEnt.getPosY()/2));
		removableBlock2 = addBlock((int)(posRight/2), (int)(playerEnt.getPosY()/2));

		sceneColour1.r = 0.0f;
		sceneColour1.g = 1.0f;
		sceneColour1.b = 0.0f;
		sceneColour1.a = 0.7f;
		sceneColour2.r = 1.0f;
		sceneColour2.g = 0.7f;
		sceneColour2.b = 0.0f;
		sceneColour2.a = 0.4f;
		sceneBorealis.setDefaultColours(sceneColour1, sceneColour2);
	}

	else if(stage == 3)
	{
		Entity* ent;
		ent = staticEntList.getHead();
		while(ent != NULL)
		{
			ent->setStatus(1.0);
			ent->setGlowColour(1.0, 1.0, 1.0, 1.0);
			ent = staticEntList.getNext();
		}
	}

	else if(stage == 4)
	{
		sceneColour1.r = 1.0f;
		sceneColour1.g = 0.0f;
		sceneColour1.b = 0.0f;
		sceneColour1.a = 0.7f;
		sceneColour2.r = 1.0f;
		sceneColour2.g = 0.7f;
		sceneColour2.b = 0.0f;
		sceneColour2.a = 0.4f;
		sceneBorealis.setDefaultColours(sceneColour1, sceneColour2);
	}
}

/*------------------------------------------------------------------------------------------------*/
/*------------------------------ Entity Collision Functions --------------------------------------*/
/*------------------------------------------------------------------------------------------------*/


template <class T> void Scene::performCollisions(T ent)
{
	DynamicEntity *current, *saved;
	Entity *staticEnt;
	Vector projection;

	/*------------Collision vs Static ents-----------*/

	int x,y;
	x= (int)( ((ent->getPosX())/2) ); 	
	y= (int)( ((ent->getPosY())/2) );

	//collide all static entities which can be found in the 8 surrounding array locations.
	for(int i =0; i<=8; i++)
	{
		switch(i)
			{
			case 1:
				staticEnt = getSceneArrayElement(x, y);
				break;
			case 0:
				staticEnt = getSceneArrayElement(x, y-1);
				break;
			case 2:
				staticEnt = getSceneArrayElement(x, y+1);
				break;
			case 3:
				staticEnt = getSceneArrayElement(x+1, y);
				break;
			case 4:
				staticEnt = getSceneArrayElement(x-1, y);
				break;
			case 5:
				staticEnt = getSceneArrayElement(x+1, y+1);
				break;
			case 6:
				staticEnt = getSceneArrayElement(x+1, y-1);
				break;
			case 7:
				staticEnt = getSceneArrayElement(x-1, y+1);
				break;
			case 8:
				staticEnt = getSceneArrayElement(x-1, y-1);
				break;
			}
		
		if(staticEnt!=NULL)
		{
			projection = detectIntersect(ent, staticEnt); 
			ent->collideResponse(projection, (T)staticEnt);
			staticEnt->collideResponse(projection, ent);
		}

	}

	/*-----------Collision vs Dynamic ents-----------*/

	saved = (DynamicEntity*) dynEntList.getCurrent(); //preserve current list position
	current = (DynamicEntity*) dynEntList.getHead();
	//DynamicEntity temp;

	while(current != NULL)
	{
		projection = detectIntersect(ent, current);
		ent->collideResponse(projection, current);
		current->collideResponse(-projection, (DynamicEntity*)ent);

		current = (DynamicEntity*) dynEntList.getNext();
	}

	dynEntList.setEnt(saved); //restore list pointer to original position.
}


Vector Scene::detectIntersect(Entity* A, Entity* B)
{	

	Vector intersect;
	Vector tempA, tempB;
	//float tempA_size, tempB_size;

	intersect.x=0;
	intersect.y=0;

	/*-------------Check entities are valid-------------*/

	if(A == NULL || B == NULL)
	{
		return intersect;
	}

	if( A == B )
	{
		return intersect;
	}
		
	tempA.x = A->getPosX(); 
	tempA.y = A->getPosY(); 
	tempB.x = B->getPosX(); 
	tempB.y = B->getPosY();

	/*Stage wrap check- objects close to stage boundary must be moved before collision detection begins.*/

	if(	(A->getPosX() > ((sceneWidth - A->getSize())*2))
	||	(B->getPosX() > ((sceneWidth - B->getSize())*2)) ) //If either object is near right scene boundary...
	{
		A->incrPosX( (float)-sceneWidth ); //move both objects left 1/2 of stage width.
		B->incrPosX( (float)-sceneWidth );
	}

	else if((A->getPosX() < A->getSize()*2)
	||		(B->getPosX() < B->getSize()*2) )	//If either object is near left scene boundary...
	{
		A->incrPosX( (float)sceneWidth ); //move both objects right 1/2 of stage width.
		B->incrPosX( (float)sceneWidth );
	}

	if( (A->getPosY() > ((sceneHeight-1)*2))
	||	(B->getPosY() > ((sceneHeight-1)*2)) ) //If either object is near upper scene boundary...
	{
		A->incrPosY( (float)-sceneHeight ); //move both objects down 1/2 of stage height.
		B->incrPosY( (float)-sceneHeight );
	}
	else if((A->getPosY() < A->getSize()*2) //If either object is near lower scene boundary...
	||		(B->getPosY() < B->getSize()*2) )
	{
		A->incrPosY( (float)sceneHeight ); //move both objects up 1/2 of stage height.
		B->incrPosY( (float)sceneHeight );
	}

	wrapEntPosition(A);
	wrapEntPosition(B);


	/*----------Check for bbox collisions------------------*/

	if(		((A->getPosX() + A->getSize()) < (B->getPosX() - B->getSize()))		//is right edge of A < left edge of B
	||		((A->getPosX() - A->getSize()) > (B->getPosX() + B->getSize()))		//is left edge of A > right edge of B
	||		((A->getPosY() + A->getSize()) < (B->getPosY() - B->getSize()))		//is top of A < base of B
	||		((A->getPosY() - A->getSize()) > (B->getPosY() + B->getSize()))  )	//is base of A < left edge of B
	{
		/*Objects are outside of their maximum bounding box.*/
			A->setPosition(tempA);
			B->setPosition(tempB);
			return intersect; 
	}


	/*----------Calculating projection vector------------*/

	/* Collision has occurred: Finding the shortest projection vector on X and Y */

	intersect.x = (A->getPosX() + A->getSize()) - (B->getPosX() - B->getSize()); //right side of A - left side of B
	intersect.y = (A->getPosY() + A->getSize()) - (B->getPosY() - B->getSize()); //top of A - base of B

	 //replace intersect.x if its magnitude is greater than the mirrored equivalent
	if(fabs(intersect.x) > fabs( ((A->getPosX() - A->getSize()) - (B->getPosX() + B->getSize())) ) )
	{
		intersect.x = ((A->getPosX() - A->getSize()) - (B->getPosX() + B->getSize()));
	}

	 //replace intersect.y if its magnitude is greater than the mirrored equivalent	
	if(fabs(intersect.y) > fabs( ((A->getPosY() - A->getSize()) - (B->getPosY() + B->getSize())) ) )
	{
		intersect.y = ((A->getPosY() - A->getSize()) - (B->getPosY() + B->getSize()));
	}

	//select the smallest intersect axis: either X or Y
	if( fabs(intersect.x) > fabs(intersect.y) )
	{
		intersect.x=0;	
	}
	else
		intersect.y=0;

	//remove stagewrap position adjustments
	A->setPosition(tempA);
	B->setPosition(tempB);
	return intersect;

}


