/***********************************************************************************/
/*                                                                                 */
/*                         Scene Class Declaration                                 */
/*                                                                                 */
/***********************************************************************************/

/*
The scene class is responsible for creating, storing and managing all of the 
individual entities which appear in the game. It also stores data pertaining
to the scene in general, such as the dimensions and the active colour scheme.
*/

#ifndef SCENE_H
#define SCENE_H

#include "list.h"
#include "entity.h"
#include "dynamicEntity.h"
#include "mouseEntity.h"
#include "playerEntity.h"
#include "sparkEntity.h"
#include "bloomEntity.h"
#include "cloudEntity.h"
#include "cameraEntity.h"
#include "starEntity.h"
#include "borealis.h"

 

class Scene
{

public:

	/*-------- Constructor & Destructor --------*/

	Scene(int scenePreset, int windowWidth_, int windowHeight_);
	~Scene();


	/*----------- Accessor Functions -----------*/

	linkedList* getDynEntList();
	linkedList* getStaticEntList();
	linkedList* getDecalEntList();
	linkedList* getBackgroundEntList();

	DynamicEntity* getPlayerEnt();
	CameraEntity* getCameraEnt();
	DynamicEntity* getBackgroundCamera();
	MouseEntity* getMouseEnt();
	Entity* getFirstDynEnt();
	Entity* getNextDynEnt();
	Entity* getFirstStaticEnt();
	Entity* getNextStaticEnt();
	Entity* getFirstDecal();
	Entity* getNextDecal();
	Entity* getFirstBackgroundEnt();
	Entity* getNextBackgroundEnt();

	Entity*	getSceneArrayElement(int, int);

	int getTotalEnts();
	int getSceneWidth();
	int getSceneHeight();

	int getWindowHeight();
	void setWindowHeight(int);
	int getWindowWidth();
	void setWindowWidth(int);

	Borealis* getBorealis();


	/*---------- Adding & Removing Scene Entities ----------*/

	int addDynEnt(Entity*);
	int addStaticEnt(Entity*);
	Entity* addBlock(int, int);
	int addDecalEnt(Entity*);
	int addBackgroundEnt(Entity*);
	void addTrail(DynamicEntity*);
	bool deleteDynEnt(Entity*);
	void deleteStaticEnt(Entity*);


	/*------------- Scene Update Functions ---------------*/

	void updateScene();
	template <class T> void performCollisions(T ent);
	void setStage(int);

	Vector detectIntersect(Entity*, Entity*);
	void wrapEntPosition(Entity*);


	/*------------- Attributes ---------------*/

private:
	//'Special' entities
	PlayerEntity playerEnt; //Player-controlled entity.
	CameraEntity cameraEnt; //Entity which defines the camera position.
	DynamicEntity backgroundCamera;
	MouseEntity *mouseEnt; //Worldspace entity which follows the mouse pointer.

	//Other persistent entities
	Entity* glowLine[20];
	int glowLineLength;
	Entity* removableBlock1, *removableBlock2;

	//Entity storage lists
	linkedList dynEntList;
	linkedList staticEntList;
	linkedList decalEntList;
	linkedList backgroundEntList;

	//Entity storage array
	EntityPtr** sceneArray;

	int sceneWidth;
	int sceneHeight;
	int windowWidth;
	int windowHeight;
	int stage;
	int starCount;

	colour sceneColour1;
	colour sceneColour2;

	Borealis sceneBorealis; 


};

#endif